package pl.webnet.JaTrac;

import java.awt.*;

class GradientColorMap extends Texture
{
	Spectrum u0v0, u1v0, u0v1, u1v1;

	public GradientColorMap()
	{
		super();
	}

	public GradientColorMap(Spectrum c0, Spectrum c1, Spectrum c2, Spectrum c3)
	{
		u0v0 = c0;
		u1v0 = c1;
		u0v1 = c2;
		u1v1 = c3;
	}

	public Spectrum updateSpectrum(Spectrum s, Point uv)
	{
		double r, g, b;
		double r0, g0, b0;
		double r1, g1, b1;
		double[] c0, c1, c2, c3;
		double u = uv.x;
		double v = uv.y;

		u *= 2;
		if(u > 1.)
			u = 2 - u;
			
		c0 = u0v0.getRGB(null);
		c1 = u1v0.getRGB(null);
		c2 = u0v1.getRGB(null);
		c3 = u1v1.getRGB(null);

		// interpolate along top edge
		r0 = c1[0] * u + c0[0] * (1 - u);
		g0 = c1[1] * u + c0[1] * (1 - u);
		b0 = c1[2] * u + c0[2] * (1 - u);

		// interpolate along bottom edge
		r1 = c3[0] * u + c2[0] * (1 - u);
		g1 = c3[1] * u + c2[1] * (1 - u);
		b1 = c3[2] * u + c2[2] * (1 - u);

		// interpolate along v dimension
		r = r1 * (1 - v) + r0 * v;
		g = g1 * (1 - v) + g0 * v;
		b = b1 * (1 - v) + b0 * v;


		return s.create(r,g,b);
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"name", "name", "",
			"u0v0", "u0v0", "",
			"u1v0", "u1v0", "",
			"u0v1", "u0v1", "",
			"u1v1", "u1v1", ""
		};

		return params;
	}
	
	public String[] getAttachArray() { return null; }

	public Object initialize()
	{
		return this;
	}

	public int getProperties()
	{
		return TEXTURE_SPECTRUM;
	}

}
