package pl.webnet.JaTrac;

class SphericalMapping extends Mapping implements XmlSettable
{
	final double recPI = 1 / Math.PI;
	final double recTwoPI = 1 / (2 * Math.PI);
	Point center;
	Vec sPole, sEquator;
	Vec crossSpSe;

	public SphericalMapping()
	{
	}

	public SphericalMapping(Point center, Vec sp, Vec se)
	{
		sPole = new Vec(sp);
		sEquator = new Vec(se);

		initialize();
	}

	public Object initialize()
	{
		//crossSpSe = sPole.cross(sEquator);
		sPole.unit();
		sEquator.unit();
		crossSpSe = sEquator.cross(sPole);

		return this;
	}

	public String[] getParamArray()
	{
		final String params[] = {
			"origin", "center", "",
			"pole", "sPole", "",
			"equator", "sEquator", "",
		};
	
		return ParamHelper.mergeParams(super.getParamArray(), params);
	}

	public String[] getAttachArray() { return null; }

	public Point getUV(Point hitPoint, Vec normal)
	{
		double u, v;
		double lat, lon;
		
		lat = Math.acos(-normal.dot(sPole));
		v = lat * recPI;

		if(v < Ray.EPSILON || Math.abs(1 - v) < Ray.EPSILON)
			u = 0.;
		else
		{
			double ratio = sEquator.dot(normal) / Math.sin(lat);

			if(ratio < -1)
				ratio = -1;
			else
				if(ratio > 1)
					ratio = 1;

			lon = Math.acos(ratio) * recTwoPI;
			
			if(crossSpSe.dot(normal) > 0)
				u = lon;
			else
				u = 1 - lon;
		}

		return new Point(u, v, 0.);
	}
}
