package pl.webnet.JaTrac;

import java.util.*;

final class UniformGrid extends IntersectionTester implements XmlSettable
{
	Scene scene;
	BoundingBox sceneBB = new BoundingBox();
	int width = 20, height = 20 , depth = 20;
	double scaleWidth, scaleHeight, scaleDepth;
	Voxel[][][] voxelGrid;

	public void initialize(Scene scene, Camera cam)
	{
		this.scene = scene;
		voxelGrid = new Voxel[width][height][depth];

		BoundingBox tempBx = new BoundingBox();

//		System.out.print("Measuring scene bounding box...");
		for(Iterator i = scene.iterator(); i.hasNext(); )
		{
			Primitive prim = (Primitive)i.next();

			sceneBB.swallow(prim.getBoundingBox(tempBx));

//			System.out.println("Object: " + prim);
		}

//		System.out.println("\b\b\b: " + sceneBB);

		System.out.print("Filling voxel grid with object pointers...");
		scaleWidth = 1.001 * (sceneBB.x1 - sceneBB.x0) / width;
		scaleHeight = 1.001 * (sceneBB.y1 - sceneBB.y0) / height;
		scaleDepth = 1.001 * (sceneBB.z1 - sceneBB.z0) / depth;

		for(Iterator i = scene.iterator(); i.hasNext(); )
		{
			Primitive prim = (Primitive)i.next();

			assignToVoxels(prim);
		}

		System.out.println("\b\b\b: " + Voxel.count + " voxels used (out of " + width * height * depth + ")");
	}

	private void assignToVoxels(Primitive prim)
	{
		BoundingBox bx = new BoundingBox();
		Voxel currentVoxel;

		prim.getBoundingBox(bx);

		// find range of vertexes that primitives' bounding box spans
		int x0 = (int)((bx.x0 - sceneBB.x0) / scaleWidth);
		int x1 = (int)((bx.x1 - sceneBB.x0) / scaleWidth);
		int y0 = (int)((bx.y0 - sceneBB.y0) / scaleHeight);
		int y1 = (int)((bx.y1 - sceneBB.y0) / scaleHeight);
		int z0 = (int)((bx.z0 - sceneBB.z0) / scaleDepth);
		int z1 = (int)((bx.z1 - sceneBB.z0) / scaleDepth);

		// 
		for(int x = x0; x <= x1; x++)
			for(int y = y0; y <= y1; y++)
				for(int z = z0; z <= z1; z++)
				{
					if(voxelGrid[x][y][z] == null)
					{
						voxelGrid[x][y][z] = new Voxel();

						voxelGrid[x][y][z].boundingBox.set(x * scaleWidth + sceneBB.x0 - Ray.EPSILON, (x + 1) * scaleWidth + sceneBB.x0,
																								y * scaleHeight + sceneBB.y0 - Ray.EPSILON, (y + 1) * scaleHeight + sceneBB.y0,
																								z * scaleDepth + sceneBB.z0 - Ray.EPSILON, (z + 1) * scaleDepth + sceneBB.z0);
					}

					voxelGrid[x][y][z].attachPrimitive(prim);
				}

	//	System.out.println("o: " + x0 + ", " + x1 + ", " + y0 + ", " + y1 + ", " + z0 + ", " + z1);

	}


	public int intersectRay(Ray ray)
	{
		return intersect(ray, scene.rootNode, 0, false);
	}

	public int intersectShadowfeeler(Ray ray)
	{
		return intersect(ray, scene.rootNode, 0, true);
	}


	public int intersect(Ray ray, Level level, int hits, boolean shadow)
	{
		Point startPoint;

		if(sceneBB.contains(ray.origin))
			startPoint = new Point(ray.origin.x, ray.origin.y, ray.origin.z);
		else
		{
			double t = sceneBB.intersect(ray);

			if(t < 0)
				return 0;

			startPoint = new Point(ray.origin.x + ray.x * t,
															ray.origin.y + ray.y * t,
															ray.origin.z + ray.z * t);

		}
/*
		voxelTraverser = init(startPoint, ray);

		while(voxelTraverser.traverse())
			intersectVoxel(ray, level, hits, shadow);
*/
			return init(startPoint, ray, shadow);
		
	}

	public int init(Point p, Ray ray, boolean shadow)
	{
		double dx, dy, dz;
		int x, y, z;
		int sx, sy, sz;
		double deltaX = 0, deltaY = 0, deltaZ = 0;
		int px = 0, py = 0, pz = 0;
		int hits = 0;

		x = (int)((p.x - sceneBB.x0) / scaleWidth);
		y = (int)((p.y - sceneBB.y0) / scaleHeight);
		z = (int)((p.z - sceneBB.z0) / scaleDepth);

		// quickly check if there's an intersection in the current voxel
		// so that we can avoid initializing the 3DDDA in case there is.
		if(voxelGrid[x][y][z] != null)
		{
			hits = intersectVoxelContents(ray, voxelGrid[x][y][z], shadow);

			if(hits != 0)
				return hits;
		}
	
		if(ray.x == 0)
		{
			dx = Double.MAX_VALUE;
			sx = 0;
		}
		else
		{
			if(ray.x > 0)
			{
				px = 1;
				dx = ((x + 1) * scaleWidth + sceneBB.x0 - p.x) / ray.x;
				sx = width;
				deltaX = scaleWidth / ray.x;
			}
			else
			{
				px = -1;
				dx = (p.x - (x * scaleWidth + sceneBB.x0)) / -ray.x;
				sx = -1;
				deltaX = scaleWidth / -ray.x;
			}
		}

		if(ray.y == 0)
		{
			dy = Double.MAX_VALUE;
			sy = 0;
		}
		else
		{
			if(ray.y > 0)
			{
				py = 1;
				dy = ((y + 1) * scaleHeight + sceneBB.y0 - p.y) / ray.y;
				sy = height;
				deltaY = scaleHeight / ray.y;
			}
			else
			{
				py = -1;
				dy = (p.y - (y * scaleHeight + sceneBB.y0)) / -ray.y;
				sy = -1;
				deltaY = -scaleHeight / ray.y;
			}
		}

		if(ray.z == 0)
		{
			dz = Double.MAX_VALUE;
			sz = 0;
		}
		else
		{
			if(ray.z > 0)
			{
				pz = 1;
				dz = ((z + 1) * scaleDepth + sceneBB.z0 - p.z) / ray.z;
				sz = depth;
				deltaZ = scaleDepth / ray.z;
			}
			else
			{
				pz = -1;
				dz = (p.z - (z * scaleDepth + sceneBB.z0)) / -ray.z;
				sz = -1; 
				deltaZ = -scaleDepth / ray.z;
			}
		}

		do
		{
			if(dx <= dy && dx <= dz)
			{
				x += px;
				if(x == sx)
					break;
				dx += deltaX;
			}
			else
				if(dy <= dx && dy <= dz)
				{
					y += py;
					if(y == sy)
						break;
					dy += deltaY;
				}
				else
					if(dz <= dy && dz <= dx)
					{
						z += pz;
						if(z == sz)
							break;
						dz += deltaZ;
					}

			if(voxelGrid[x][y][z] != null)
			{
				hits = intersectVoxelContents(ray, voxelGrid[x][y][z], shadow);

				if(hits != 0)
					return hits;
			}

		} while(true);

		return 0;
	}

	public int intersectVoxelContents(Ray ray, Voxel voxel, boolean shadowRay)
	{
		int allHits = 0;
		int hits = 0;

		for(Iterator i = voxel.prims.iterator(); i.hasNext(); )
		{
			Primitive primitive = (Primitive)i.next();
			
			if((shadowRay && primitive.castShadows == true) ||
					(!shadowRay && (ray.generation == 0 || primitive.reflected == true)))
			{
				// if current ray's intersection with this object was computed
				// during visit in some other voxel, reuse intersection point then
				// obtained.
				if(primitive.mailbox == ray.serialId)
				{
					if(primitive.lastIntersection != null)
					{
						if(voxel.boundingBox.contains(primitive.lastHitPoint))
						{
							ray.intersections.add(primitive.lastIntersection);
							hits = 1;
						}
					}
				}
				else
				{
					hits = ((Primitive)primitive).intersect(ray);
					primitive.mailbox = ray.serialId;

					// make sure the intersection occured in the current voxel
					if(hits > 0)
					{
						Intersection intersection = (Intersection)ray.intersections.first();
						Point hitPoint = ray.rayPoint(intersection.t);

						// not in current voxel? store the intersection information
						// for future use together with ray ID in primitive's mailbox
						if(voxel.boundingBox.contains(hitPoint) == false)
						{
							ray.intersections.remove(intersection);
							primitive.lastIntersection = intersection;
							primitive.lastHitPoint = hitPoint;

							hits = 0;
						}
					}
					else
					{
						// remember that there are no intersections with this
						// primitive, so that we do not waste time in following
						// voxels
						primitive.lastIntersection = null;
					}
				}

				allHits += hits;
			}
		}

		return allHits;
	}
	
	public Object initialize()
	{
		return this;
	}

	public String[] getParamArray()
	{
		String params[] = {
			"width", "width", "",
			"height", "height", "",
			"depth", "depth", ""
		};

		return params;
	}

	public String[] getAttachArray()
	{
		String params[] = {
		};

		return params;
	}


}
	
