/* $Id: integra.jflex,v 1.4 2001/06/03 01:42:38 thorgal Exp $ */
package pl.webnet.JaTrac;
import java.util.*;

%%

%class IntegraLexer
%caseless
%line
%column
%int

%{

private double[] dbl;
private Camera cam;
private int triangles, vertices, cnt, blocks = 0;
private Point[] verts;

private double[] parseDoubles(String str, int count)
{
	StringTokenizer st = new StringTokenizer(str);
	double dbl[] = new double[count];

	for(int i = 0; i < count; i++)
	{
		dbl[i] = Double.parseDouble(st.nextToken());
//		System.out.print(dbl[i] + " ");
	}
//	System.out.println("");

	return dbl;
}

%}

NUM = "-"? [0-9]+ ("." [0-9]+)?
NL = \n | \r | \r\n
WS = ([\r|\n|\r\n|]|[ \t\f])*
Identifier = [:jletter:] [:jletterdigit:]*

%state VIEWPOINT
%state SCREEN
%state RESOLUTION
%state ATTR

%state VERTICES
%state TRIANGLES

%%
<YYINITIAL> "viewpoint" { yybegin(VIEWPOINT); }
<YYINITIAL> "screen" { yybegin(SCREEN); }
<YYINITIAL> "resolution" { yybegin(RESOLUTION); }
<YYINITIAL> "Attr"{WS}{Identifier} {
	String name = yytext().substring("Attr ".length());
	System.out.println("Attr name: " + name);
	yybegin(ATTR); 
}
<ATTR> "enddef" { yybegin (YYINITIAL); }

<YYINITIAL> "vertices"{WS}{NUM} {
		dbl = parseDoubles(yytext().substring("vertices".length() + 1), 1);

		vertices = (int)dbl[0];
		System.out.println("Vertices: " + vertices);
		cnt = 0;
		verts = new Point[vertices];
		yybegin(VERTICES);
	}

<VERTICES> ({NUM}{WS}){3} {
	dbl = parseDoubles(yytext(), 3);
	verts[cnt++] = new Point(dbl[0] / 100, dbl[1] / 100, dbl[2] / 100);

	if(cnt == vertices)
		yybegin(YYINITIAL);
}

<YYINITIAL> "triangles"{WS}{NUM} {
		dbl = parseDoubles(yytext().substring("triangles".length() + 1), 1);

		triangles = (int)dbl[0];
		System.out.println("Triangles: " + triangles);
		cnt = 0;
		yybegin(TRIANGLES);
	}

<TRIANGLES> ({NUM}{WS}){3} {
	dbl = parseDoubles(yytext(), 3);

	if(blocks != 1)
	{
		Triangle t = new Triangle(verts[(int)dbl[0]], verts[(int)dbl[1]], verts[(int)dbl[2]]);
		((TreeNode)t).setParent(Loader.scene.rootNode);

		Loader.scene.add(t);
	}

	if(++cnt == triangles)
	{
		yybegin(YYINITIAL);

		++blocks;
	}
}
<YYINITIAL> "camera"{WS}{Identifier} {
		System.out.println("camera: " + yytext());
		cam = new Camera();
		cam.name = yytext();

		Loader.scene.setCamera(cam);
		Loader.scene.setSpectrum(new SpectrumRGB(Spectrum.white));
		
		Surface ds = new Surface("default", 0.76, 1.0, 0.1, 0.24, 0.0, 0.0, 1.0);
		Mapping dm = new Mapping();
		dm.surface = ds;

		Loader.scene.add(dm);

		Loader.scene.add(new PointLight(new Point(-2, 4, 0), new SpectrumRGB(Spectrum.white)));
			
	}

<VIEWPOINT> {
({NUM}{WS}){3} {
		yybegin(YYINITIAL);

		dbl = parseDoubles(yytext(), 3);
		cam.origin = new Point(dbl);
	}
}

<SCREEN> {
({NUM}{WS}){9} {
		yybegin(YYINITIAL);

		dbl = parseDoubles(yytext(), 9);
		cam.imagePlaneUL = new Point(dbl[0], dbl[1], dbl[2]);
		cam.imagePlaneUR = new Point(dbl[3], dbl[4], dbl[5]);
		cam.imagePlaneLL = new Point(dbl[6], dbl[7], dbl[8]);
	}
}

<RESOLUTION> {
({NUM}{WS}){2} {
		yybegin(YYINITIAL);

		dbl = parseDoubles(yytext(), 2);

		cam.attachScreen(new Screen((int)(dbl[0] * 1), (int)(dbl[1] * 1)));
		//cam.attachScreen(new Screen(320, 240));
	}
}

.|\n  { /* error */ }

